searchData={"items":[{"type":"module","title":"Carve","doc":"Carve simplifies JSON API development in Phoenix by automatically formatting endpoint outputs with associated links. Define resource relationships with a simple DSL, and let Carve handle link inclusion and query-based filtering, reducing boilerplate and ensuring consistent, flexible API responses.\n\nIt provides functionality for:\n- Rendering JSON endpoints with links automatically\n- Creating index/show methods for views from a DSL\n- Retrieving links between different data types\n- Encoding and decoding IDs using HashIds\n- Configuring the application","ref":"Carve.html"},{"type":"function","title":"Carve.configure/1","doc":"Configures Carve with the given arguments.\n\nThis function is typically called during application start, but can also be used\nto reconfigure Carve at runtime.","ref":"Carve.html#configure/1"},{"type":"function","title":"Parameters - Carve.configure/1","doc":"- `args`: A keyword list of configuration options.","ref":"Carve.html#configure/1-parameters"},{"type":"function","title":"Examples - Carve.configure/1","doc":"iex> Carve.configure(salt: \"my_salt\", min_length: 8)\n    :ok","ref":"Carve.html#configure/1-examples"},{"type":"function","title":"Carve.decode/1","doc":"Decodes a hash without a type using HashIds.\n\nThis function attempts to decode the hash without knowing its type.","ref":"Carve.html#decode/1"},{"type":"function","title":"Parameters - Carve.decode/1","doc":"- `hash`: The string hash to decode.","ref":"Carve.html#decode/1-parameters"},{"type":"function","title":"Returns - Carve.decode/1","doc":"The decoded integer ID.","ref":"Carve.html#decode/1-returns"},{"type":"function","title":"Examples - Carve.decode/1","doc":"iex> Carve.decode(\"abc123def\")\n    {:ok, 123}","ref":"Carve.html#decode/1-examples"},{"type":"function","title":"Carve.decode/2","doc":"Decodes a hash with a given type using HashIds.","ref":"Carve.html#decode/2"},{"type":"function","title":"Parameters - Carve.decode/2","doc":"- `type`: The type of the ID (as an atom).\n- `hash`: The string hash to decode.","ref":"Carve.html#decode/2-parameters"},{"type":"function","title":"Returns - Carve.decode/2","doc":"The decoded integer ID.","ref":"Carve.html#decode/2-returns"},{"type":"function","title":"Examples - Carve.decode/2","doc":"iex> Carve.decode(:user, \"abc123def\")\n    {:ok, 123}","ref":"Carve.html#decode/2-examples"},{"type":"function","title":"Carve.encode/2","doc":"Encodes an ID for a given type using HashIds.","ref":"Carve.html#encode/2"},{"type":"function","title":"Parameters - Carve.encode/2","doc":"- `type`: The type of the ID (as an atom).\n- `id`: The integer ID to encode.","ref":"Carve.html#encode/2-parameters"},{"type":"function","title":"Returns - Carve.encode/2","doc":"A string representing the encoded ID.","ref":"Carve.html#encode/2-returns"},{"type":"function","title":"Examples - Carve.encode/2","doc":"iex> Carve.encode(:user, 123)\n    \"abc123def\"","ref":"Carve.html#encode/2-examples"},{"type":"function","title":"Carve.fetch_include/1","doc":"Fetches and parses the include parameter from the params map.\n\nThis function determines if the include parameter was specified and parses it accordingly.","ref":"Carve.html#fetch_include/1"},{"type":"function","title":"Parameters - Carve.fetch_include/1","doc":"- `params`: The full params map from the controller.","ref":"Carve.html#fetch_include/1-parameters"},{"type":"function","title":"Returns - Carve.fetch_include/1","doc":"- `nil`: If the parameter was not specified (include everything).\n- `[]`: If an empty string was passed.\n- `[atom]`: A list of atoms representing the types to include.","ref":"Carve.html#fetch_include/1-returns"},{"type":"function","title":"Examples - Carve.fetch_include/1","doc":"iex> Carve.fetch_include(%{\"include\" => \"foo,bar\"})\n    [:foo, :bar]\n\n    iex> Carve.fetch_include(%{\"include\" => \"\"})\n    []\n\n    iex> Carve.fetch_include(%{})\n    nil","ref":"Carve.html#fetch_include/1-examples"},{"type":"function","title":"Carve.include_param_specified?/1","doc":"Determines if the include parameter was specified in the query string.","ref":"Carve.html#include_param_specified?/1"},{"type":"function","title":"Parameters - Carve.include_param_specified?/1","doc":"- `params`: The full params map from the controller.","ref":"Carve.html#include_param_specified?/1-parameters"},{"type":"function","title":"Returns - Carve.include_param_specified?/1","doc":"- `true`: If the include parameter was specified (even if empty).\n- `false`: If the include parameter was not specified at all.","ref":"Carve.html#include_param_specified?/1-returns"},{"type":"function","title":"Examples - Carve.include_param_specified?/1","doc":"iex> Carve.include_param_specified?(%{\"include\" => \"foo,bar\"})\n    true\n\n    iex> Carve.include_param_specified?(%{\"include\" => \"\"})\n    true\n\n    iex> Carve.include_param_specified?(%{})\n    false","ref":"Carve.html#include_param_specified?/1-examples"},{"type":"function","title":"Carve.links/2","doc":"Retrieves links for a given module and data or ID(s).\n\nThis function supports various input types:\n- Single integer ID\n- List of integer IDs\n- Single map (data structure)\n- List of maps (data structures)","ref":"Carve.html#links/2"},{"type":"function","title":"Parameters - Carve.links/2","doc":"- `module`: The module to retrieve links for.\n- `data_or_ids`: The data or ID(s) to retrieve links for.","ref":"Carve.html#links/2-parameters"},{"type":"function","title":"Returns - Carve.links/2","doc":"A list of link maps, each containing `:type`, `:id`, and `:data` keys.","ref":"Carve.html#links/2-returns"},{"type":"function","title":"Examples - Carve.links/2","doc":"iex> Carve.links(PostJSON, 1)\n    [%{type: :user, id: \"abc123\", data: %{...}}, %{type: :comment, id: \"def456\", data: %{...}}]\n\n    iex> Carve.links(PostJSON, [1, 2, 3])\n    [%{type: :user, id: \"abc123\", data: %{...}}, %{type: :comment, id: \"def456\", data: %{...}}, ...]\n\n    iex> Carve.links(PostJSON, %{id: 1, title: \"Test Post\"})\n    [%{type: :user, id: \"abc123\", data: %{...}}, %{type: :comment, id: \"def456\", data: %{...}}]","ref":"Carve.html#links/2-examples"},{"type":"function","title":"Carve.start/2","doc":"Starts the Carve application.\n\nThis function is called automatically by the OTP application behaviour.\nIt configures Carve using the configuration returned by `Carve.Config.get/0`.","ref":"Carve.html#start/2"},{"type":"function","title":"Returns - Carve.start/2","doc":"- `{:ok, pid}`: The pid is the process identifier of the started application.","ref":"Carve.html#start/2-returns"},{"type":"module","title":"Carve.Config","doc":"Handles configuration for Carve.","ref":"Carve.Config.html"},{"type":"function","title":"Carve.Config.get/0","doc":"Retrieves the configuration for Carve.","ref":"Carve.Config.html#get/0"},{"type":"function","title":"Carve.Config.get/2","doc":"Gets a specific configuration value.","ref":"Carve.Config.html#get/2"},{"type":"module","title":"Carve.HashIds","doc":"Carve.HashIds provides functionality for encoding and decoding IDs using HashIds.\n\nThis module is responsible for:\n- Configuring the HashIds settings\n- Encoding integer IDs into hashed strings\n- Decoding hashed strings back into integer IDs\n- Handling entity-specific salting for added security\n\nIt uses the Hashids library internally and provides a convenient API for Carve users.","ref":"Carve.HashIds.html"},{"type":"function","title":"Carve.HashIds.configure/1","doc":"Configures the HashIds settings.\n\nThis function sets up the salt and minimum length for HashIds encoding.\nIt stores these settings in persistent term storage for efficient access.","ref":"Carve.HashIds.html#configure/1"},{"type":"function","title":"Parameters - Carve.HashIds.configure/1","doc":"- `opts`: A keyword list of configuration options.\n  - `:salt` - The salt used for hashing (default: 1207:Rumi)\n  - `:min_length` - The minimum length of generated hashes (default: 4)","ref":"Carve.HashIds.html#configure/1-parameters"},{"type":"function","title":"Example - Carve.HashIds.configure/1","doc":"iex> Carve.HashIds.configure(salt: \"my_custom_salt\", min_length: 8)\n    :ok","ref":"Carve.HashIds.html#configure/1-example"},{"type":"function","title":"Carve.HashIds.decode/1","doc":"Decodes a hashed string without specifying the entity type.\n\nThis function attempts to decode the hash without verifying the entity type.","ref":"Carve.HashIds.html#decode/1"},{"type":"function","title":"Parameters - Carve.HashIds.decode/1","doc":"- `hash`: The hashed string to be decoded","ref":"Carve.HashIds.html#decode/1-parameters"},{"type":"function","title":"Returns - Carve.HashIds.decode/1","doc":"`{:ok, id}` if decoding is successful, where `id` is the original integer ID.\n`{:error, reason}` for decoding errors.","ref":"Carve.HashIds.html#decode/1-returns"},{"type":"function","title":"Example - Carve.HashIds.decode/1","doc":"iex> Carve.HashIds.decode(\"Xk9Lp2Rr4m\")\n    {:ok, 123}","ref":"Carve.HashIds.html#decode/1-example"},{"type":"function","title":"Carve.HashIds.decode/2","doc":"Decodes a hashed string back into an integer ID for a given type.","ref":"Carve.HashIds.html#decode/2"},{"type":"function","title":"Parameters - Carve.HashIds.decode/2","doc":"- `type`: An atom representing the entity type (e.g., :user, :post)\n- `hash`: The hashed string to be decoded","ref":"Carve.HashIds.html#decode/2-parameters"},{"type":"function","title":"Returns - Carve.HashIds.decode/2","doc":"`{:ok, id}` if decoding is successful, where `id` is the original integer ID.\n`{:error, :invalid_entity_type}` if the decoded salt doesn't match the given type.\n`{:error, reason}` for other decoding errors.","ref":"Carve.HashIds.html#decode/2-returns"},{"type":"function","title":"Example - Carve.HashIds.decode/2","doc":"iex> Carve.HashIds.decode(:user, \"Xk9Lp2Rr4m\")\n    {:ok, 123}","ref":"Carve.HashIds.html#decode/2-example"},{"type":"function","title":"Carve.HashIds.encode/2","doc":"Encodes an integer ID for a given type into a hashed string.","ref":"Carve.HashIds.html#encode/2"},{"type":"function","title":"Parameters - Carve.HashIds.encode/2","doc":"- `type`: An atom representing the entity type (e.g., :user, :post)\n- `id`: The integer ID to be encoded","ref":"Carve.HashIds.html#encode/2-parameters"},{"type":"function","title":"Returns - Carve.HashIds.encode/2","doc":"A string representing the encoded ID.","ref":"Carve.HashIds.html#encode/2-returns"},{"type":"function","title":"Example - Carve.HashIds.encode/2","doc":"iex> Carve.HashIds.encode(:user, 123)\n    \"Xk9Lp2Rr4m\"","ref":"Carve.HashIds.html#encode/2-example"},{"type":"module","title":"Carve.Links","doc":"Carve.Links provides functionality for retrieving and processing links between different entities.\n\nThis module is responsible for:\n- Fetching links for entities based on their IDs or data structures\n- Handling circular references and preventing infinite loops\n- Normalizing and preparing the final result set of links\n\nIt works in conjunction with the view modules created using Carve.View to generate\na complete set of links for any given entity or set of entities.","ref":"Carve.Links.html"},{"type":"function","title":"Carve.Links.filter_links/2","doc":"Filters links based on the provided whitelist.","ref":"Carve.Links.html#filter_links/2"},{"type":"function","title":"Parameters - Carve.Links.filter_links/2","doc":"- `links`: A map of links where keys are modules and values are IDs or lists of IDs\n- `whitelist`: A list of allowed link types (atoms)","ref":"Carve.Links.html#filter_links/2-parameters"},{"type":"function","title":"Returns - Carve.Links.filter_links/2","doc":"A filtered map of links.","ref":"Carve.Links.html#filter_links/2-returns"},{"type":"function","title":"Examples - Carve.Links.filter_links/2","doc":"iex> links = %{UserJSON => [1, 2], PostJSON => [3, 4]}\n    iex> Carve.Links.filter_links(links, [:user])\n    %{UserJSON => [1, 2]}","ref":"Carve.Links.html#filter_links/2-examples"},{"type":"function","title":"Carve.Links.get_links_by_data/4","doc":"Retrieves links for a given module and data or list of data.\n\nThis function handles single data structures, lists of data structures, and\nprevents circular references using a visited map. It also supports filtering links based on a whitelist.","ref":"Carve.Links.html#get_links_by_data/4"},{"type":"function","title":"Parameters - Carve.Links.get_links_by_data/4","doc":"- `module`: The module to use for fetching and processing links\n- `data`: A single data structure or a list of data structures\n- `visited`: A map to keep track of visited entities (default: %{})\n- `whitelist`: An optional list of allowed link types (default: nil)","ref":"Carve.Links.html#get_links_by_data/4-parameters"},{"type":"function","title":"Returns - Carve.Links.get_links_by_data/4","doc":"A list of prepared link maps.","ref":"Carve.Links.html#get_links_by_data/4-returns"},{"type":"function","title":"Examples - Carve.Links.get_links_by_data/4","doc":"iex> user = %{id: 1, name: \"John Doe\", team_id: 2}\n    iex> Carve.Links.get_links_by_data(UserJSON, user)\n    [%{type: :team, id: \"abc123\", data: %{...}}, %{type: :profile, id: \"def456\", data: %{...}}]\n\n    iex> user = %{id: 1, name: \"John Doe\", team_id: 2}\n    iex> Carve.Links.get_links_by_data(UserJSON, user, %{}, [:team])\n    [%{type: :team, id: \"abc123\", data: %{...}}]","ref":"Carve.Links.html#get_links_by_data/4-examples"},{"type":"function","title":"Carve.Links.get_links_by_id/4","doc":"Retrieves links for a given module and ID or list of IDs.\n\nThis function handles single IDs, lists of IDs, and prevents circular references\nusing a visited map. It also supports filtering links based on a whitelist.","ref":"Carve.Links.html#get_links_by_id/4"},{"type":"function","title":"Parameters - Carve.Links.get_links_by_id/4","doc":"- `module`: The module to use for fetching and processing links\n- `id`: A single ID or a list of IDs\n- `visited`: A map to keep track of visited entities (default: %{})\n- `whitelist`: An optional list of allowed link types (default: nil)","ref":"Carve.Links.html#get_links_by_id/4-parameters"},{"type":"function","title":"Returns - Carve.Links.get_links_by_id/4","doc":"A list of prepared link maps.","ref":"Carve.Links.html#get_links_by_id/4-returns"},{"type":"function","title":"Examples - Carve.Links.get_links_by_id/4","doc":"iex> Carve.Links.get_links_by_id(UserJSON, 1)\n    [%{type: :team, id: \"abc123\", data: %{...}}, %{type: :profile, id: \"def456\", data: %{...}}]\n\n    iex> Carve.Links.get_links_by_id(UserJSON, [1, 2, 3], %{}, [:team])\n    [%{type: :team, id: \"abc123\", data: %{...}}]","ref":"Carve.Links.html#get_links_by_id/4-examples"},{"type":"function","title":"Carve.Links.prepare_result/2","doc":"Prepares the final result set by flattening, removing nil values, eliminating duplicates,\nand applying the whitelist filter if provided.","ref":"Carve.Links.html#prepare_result/2"},{"type":"function","title":"Parameters - Carve.Links.prepare_result/2","doc":"- `result`: A list of link maps\n- `whitelist`: An optional list of allowed link types","ref":"Carve.Links.html#prepare_result/2-parameters"},{"type":"function","title":"Returns - Carve.Links.prepare_result/2","doc":"A cleaned, uniquified, and optionally filtered list of link maps.","ref":"Carve.Links.html#prepare_result/2-returns"},{"type":"function","title":"Examples - Carve.Links.prepare_result/2","doc":"iex> result = [[%{type: :team, id: \"abc\"}, nil], [%{type: :profile, id: \"def\"}], %{type: :team, id: \"abc\"}]\n    iex> Carve.Links.prepare_result(result, [:team])\n    [%{type: :team, id: \"abc\"}]","ref":"Carve.Links.html#prepare_result/2-examples"},{"type":"module","title":"Carve.View","doc":"Carve.View provides a DSL for quickly building JSON API views in Phoenix applications.\n\nIt automatically creates `show` and `index` functions for Phoenix controllers,\nhandles ID hashing, and manages links between different entities.","ref":"Carve.View.html"},{"type":"module","title":"Usage - Carve.View","doc":"In your Phoenix JSON view module:\n\n    defmodule MyApp.UserJSON do\n      use Carve.View, :user\n\n      links fn user ->\n        %{\n          MyApp.TeamJSON => user.team_id,\n          MyApp.ProfileJSON => user.profile_id\n        }\n      end\n\n      get fn id ->\n        MyApp.Users.get_by_id!(id)\n      end\n\n      view fn user ->\n        %{\n          id: hash(user.id),\n          name: user.name,\n          team_id: MyApp.TeamJSON.hash(user.team_id),\n          profile_id: MyApp.ProfileJSON.hash(user.profile_id)\n        }\n      end\n    end\n\nThis will automatically create `show/1` and `index/1` functions that can be used\nin your Phoenix controllers, handling both data rendering and link generation.","ref":"Carve.View.html#module-usage"},{"type":"module","title":"Generated Functions - Carve.View","doc":"The `use Carve.View, :type` macro generates several functions at compile time:\n\n- `index/1`: Handles rendering a list of entities with their links.\n- `show/1`: Handles rendering a single entity with its links.\n- `hash/1`: Alias of encode_id.\n- `encode_id/1`: Encodes an ID using the view type as a salt.\n- `decode_id/1`: Decodes an ID using the view type as a salt.\n- `type_name/0`: Returns the type of the view.\n- `process_links/1`: Processes links for an entity (if `links` macro is used).\n\nHere's an example of what these functions might look like at runtime:\n\n    def index(%{result: data}) when is_list(data) do\n      results = Enum.map(data, &prepare_for_view/1)\n      links = Carve.Links.get_links_by_data(__MODULE__, data)\n      %{result: results, links: links}\n    end\n\n    def show(%{result: data}) do\n      result = prepare_for_view(data)\n      links = Carve.Links.get_links_by_data(__MODULE__, data)\n      %{result: result, links: links}\n    end\n\n    def hash(id) when is_integer(id), do: Carve.HashIds.encode(:user, id)\n    def hash(%{id: id}), do: hash(id)\n\n    def type_name, do: :user\n\n    def process_links(data) do\n      %{\n        MyApp.TeamJSON => data.team_id,\n        MyApp.ProfileJSON => data.profile_id\n      }\n    end\n\nThese generated functions work together to provide a seamless API for rendering\nJSON views with proper linking and ID hashing.","ref":"Carve.View.html#module-generated-functions"},{"type":"macro","title":"Carve.View.__using__/1","doc":"Sets up the view module with the given type.\n\nThis macro is called when you use `use Carve.View, :type` in your module.\nIt imports Carve.View functions and sets up the necessary module attributes.","ref":"Carve.View.html#__using__/1"},{"type":"macro","title":"Parameters - Carve.View.__using__/1","doc":"- `type`: An atom representing the type of the view (e.g., :user, :post).","ref":"Carve.View.html#__using__/1-parameters"},{"type":"macro","title":"Example - Carve.View.__using__/1","doc":"defmodule MyApp.UserJSON do\n      use Carve.View, :user\n      # ... rest of the module\n    end","ref":"Carve.View.html#__using__/1-example"},{"type":"macro","title":"Carve.View.get/1","doc":"Defines how to retrieve an entity by its ID.\n\nThis macro specifies a function to fetch an entity given its ID.","ref":"Carve.View.html#get/1"},{"type":"macro","title":"Parameters - Carve.View.get/1","doc":"- `func`: A function that takes an ID and returns the corresponding entity.","ref":"Carve.View.html#get/1-parameters"},{"type":"macro","title":"Example - Carve.View.get/1","doc":"get fn id ->\n      MyApp.Users.get_by_id!(id)\n    end","ref":"Carve.View.html#get/1-example"},{"type":"macro","title":"Carve.View.links/1","doc":"Defines the links for the current view.\n\nThis macro allows you to specify how to generate links for the current entity.","ref":"Carve.View.html#links/1"},{"type":"macro","title":"Parameters - Carve.View.links/1","doc":"- `func`: A function that takes the entity data and returns a map of links.","ref":"Carve.View.html#links/1-parameters"},{"type":"macro","title":"Example - Carve.View.links/1","doc":"links fn user ->\n      %{\n        MyApp.TeamJSON => user.team_id,\n        MyApp.ProfileJSON => user.profile_id\n      }\n    end","ref":"Carve.View.html#links/1-example"},{"type":"macro","title":"Carve.View.view/1","doc":"Defines how to render the view for the current entity.\n\nThis macro specifies how to format the entity data for JSON output.","ref":"Carve.View.html#view/1"},{"type":"macro","title":"Parameters - Carve.View.view/1","doc":"- `func`: A function that takes the entity data and returns a map for JSON rendering.","ref":"Carve.View.html#view/1-parameters"},{"type":"macro","title":"Example - Carve.View.view/1","doc":"view fn user ->\n      %{\n        id: hash(user.id),\n        name: user.name,\n        team_id: MyApp.TeamJSON.hash(user.team_id)\n      }\n    end","ref":"Carve.View.html#view/1-example"},{"type":"extras","title":"Carve","doc":"# Carve\n\nDSL for building JSON APIs fast. Creates endpoint views, renders linked data automatically.\n\nFeatures:\n\n* Rendering structured JSON views automatically\n* Retrieving links between different data types\n* Creating index/show methods for views\n* Hashing IDs with unique & short strings (e.g `123` -> `Xk9Lp2R`)\n\nHere's how an endpoint view generated by Carve looks like:\n\n```js\n{\n  \"result\": {\n    \"id\": \"D3Wcorr0oa\",\n    \"type\": \"user\",\n    \"data\": { ... }\n  },\n  \"links\": [\n    {\n      \"id\": \"Xk9Lp2Rr4m\",\n      \"type\": \"team\",\n      \"data\": { ... }\n    },\n    {\n      \"id\": \"Bz7Jt5Yq1n\",\n      \"type\": \"profile\",\n      \"data\": { ... }\n    }\n  ]\n}\n```","ref":"readme.html"},{"type":"extras","title":"Installation - Carve","doc":"Add `carve` to your list of dependencies in `mix.exs`:\n\n\n```elixir\ndef deps do\n  [\n    {:carve, \"~> 0.1.0\"}\n  ]\nend\n```\n\n\nTo configure custom hashing salt, add to `config.exs`:\n\n```elixir\nconfig :carve,\n  salt: \"your-secret-salt\",\n  min_length: 10\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Carve","doc":"In your Phoenix JSON view, just declare how your endpoint should look like:\n\n```elixir\ndefmodule UserJSON do\n    use Carve.View, :user\n\n    view fn user ->\n      %{\n        id: hash(user.id), # provided by Carve.View, encodes numerical ID (123) with an entity-specific salt (D3Wcorr0oa).\n        name: user.name\n      }\n    end\nend\n```\n\nAnd that's it -- this macro will make `index(%{ result: users })` and `show(%{ result: user })` methods available for your controller.\n\nOf course, just formatting the view alone is not Carvers only point; you can declare the links of each view, and Carve will automatically output them for the client:\n\n```elixir\ndefmodule UserJSON do\n    use Carve.View, :user\n\n    # Return the links of a given user as ViewModule => id\n    links fn user ->\n        %{\n            FooWeb.TeamJSON => user.team_id, # You can also pass list of ids or the Ecto record(s)\n            FooWeb. ProfileJSON => user.profile_id\n        }\n    end\n\n    # Provide a method to retrieve user by id. This will be used by Carve to render links automatically.\n    get fn id ->\n        Foo.Users.get_by_id!(id)\n    end\n\n    view fn user ->\n      %{\n        id: hash(user.id),\n        team_id: FooWeb.TeamJSON.hash(user.team_id),\n        profile_id: FooWeb.ProfileJSON.hash(user.profile_id),\n        name: user.name\n      }\n    end\nend\n```\n\nAfter activating `show` and `index` methods in the controller:\n\n```elixir\nrender(conn, :index, result: requests) # or render(conn, :show, result: record)\n```\n\nExample response Carve will render for this view:\n\n```json\n{\n  \"result\": {\n    \"id\": \"D3Wcorr0oa\",\n    \"type\": \"user\",\n    \"data\": {\n      \"id\": \"D3Wcorr0oa\",\n      \"name\": \"John Doe\",\n      \"team_id\": \"Xk9Lp2Rr4m\",\n      \"profile_id\": \"Bz7Jt5Yq1n\"\n    }\n  },\n  \"links\": [\n    {\n      \"id\": \"Xk9Lp2Rr4m\",\n      \"type\": \"team\",\n      \"data\": {\n        \"id\": \"Xk9Lp2Rr4m\",\n        \"name\": \"Engineering Team\",\n        \"description\": \"Our awesome engineering team\"\n      }\n    },\n    {\n      \"id\": \"Bz7Jt5Yq1n\",\n      \"type\": \"profile\",\n      \"data\": {\n        \"id\": \"Bz7Jt5Yq1n\",\n        \"bio\": \"Software engineer passionate about Elixir\",\n        \"avatar_url\": \"https://example.com/avatars/johndoe.jpg\"\n      }\n    }\n  ]\n}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Example Controller - Carve","doc":"Carve provides a flexible way to control which linked data is included in the response. This is achieved through the include parameter. Here's an example of how to use Carve in your Phoenix controllers:\n\n```elixir\ndefmodule UserController do\n  use FooWeb, :controller\n\n  def show(conn, %{\"id\" => id} = params) do\n    user = Foo.Users.get_user!(id)\n    include = Carve.fetch_include(params)\n\n    render(conn, :show, %{ result: user, include: include })\n  end\n\n  def index(conn, params) do\n    users = Foo.Users.list_users()\n    include = Carve.fetch_include(params)\n\n    render(conn, :index, %{ result: users, include: include })\n  end\nend\n```\n\nThis example also shows reading & parsing the `include` parameter, which can be one of following:\n\n* Not specified (`GET  /api/users`): All link types are included.\n* Empty list (`GET /api/users?included=`): No link types are included.\n* Custom types: (`GET /api/users/123?include=team,profile`): Include comma-separated link types only.","ref":"readme.html#example-controller"},{"type":"extras","title":"How does it work? - Carve","doc":"* Carve macros create view functions `index(%{ result: users })` and `show(%{ result: user })`\n* Controller calls these view functions\n* Carve pulls the list of links for given data (list or single record)\n* Carve calls the `get_by_id` (`get` macro expanded) and `prepare_for_view` (`view` macro expanded) functions for each link\n* The final expanded list of links get flattened & cleaned, returned to user with the main result: `{ result: {} || [], links: [] }`","ref":"readme.html#how-does-it-work"},{"type":"extras","title":"API - Carve","doc":"More detailed API docs are available at [https://hexdocs.pm/carve/Carve.html](https://hexdocs.pm/carve/Carve.html)","ref":"readme.html#api"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}